/*
 * Copyright 2008-2010 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package org.wamblee.glassfish.auth;

import java.util.List;

import org.wamblee.glassfish.auth.cache.api.AuthenticationCache;

/**
 * Decorator on the authentication repository that performs caching.
 *  
 * @author Erik Brakkee
 */
public class CachedAuthenticationRepository implements AuthenticationRepository {
    
    private AuthenticationCache cache; 
    private AuthenticationRepository decorated; 
    
    /**
     * Constructs the cached repository.
     * @param aCache Cache to use. 
     * @param aDecorated Decorated authentication repository.
     */
    public CachedAuthenticationRepository(AuthenticationCache aCache, AuthenticationRepository aDecorated) {
        cache = aCache; 
        decorated = aDecorated;
    }

    @Override
    public String findPassword(String aUsername) {
        String password = cache.getPassword(aUsername);
        if ( password == null ) { 
            password = decorated.findPassword(aUsername);
            if ( password != null ) { 
                cache.setPassword(aUsername, password);
            }
        }
        return password;
    }

    @Override
    public List<String> findGroups(String aUsername) {
        List<String> groups = cache.getGroups(aUsername);
        if ( groups == null ) { 
            groups = decorated.findGroups(aUsername);
            // findGroups always returns non-null. 
            cache.setGroups(aUsername, groups);
        }
        return groups;
    }

    @Override
    public String findSeed(String aUsername) {
        String seed = cache.getSeed(aUsername);
        if ( seed == null ) { 
            seed = decorated.findSeed(aUsername);
            if ( seed != null ) { 
                cache.setSeed(aUsername, seed);
            }
        }
        return seed;
    }

}
